<?php

/*
 *  Cryptophoto User plugin adds the Cryptophoto two-factor authentication
 *  to users and allows them to manage their own Cryptophoto settings
 *  through a new section in the user settings page
*/

class Cryptophoto_CryptophotoUser_Plugin extends Atmail_Controller_Plugin
{
  
  // plugin info
  protected $_pluginFullName = 'Cryptophoto User';
  protected $_pluginAuthor = 'Cryptophoto <Tech@CryptoPhoto.com>';
  protected $_pluginDescription = 'Improves security by adding the Cryptophoto two-factor authentication';
  protected $_pluginCopyright = '2012-2015 Cryptophoto';
  protected $_pluginUrl = 'http://cryptophoto.com/info/atmail-user';
  protected $_pluginNotes = '';
  protected $_pluginVersion = '1.20150610';
  protected $_pluginCompat = '6.1.5';
  protected $_pluginModule = 'mail';
  private $_loginPage = false;
  
  /*
   * class constructor
  */
  public function __construct() {
    parent::__construct();
    $this->log = Zend_Registry::get('log');
    
    if (file_exists(dirname(__FILE__) . "/CryptoPhotoUtils.php")) {
      $this->path = dirname(__FILE__) . "/CryptoPhotoUtils.php";
    } 
    else if (file_exists(APP_ROOT . "application/modules/mail/plugins/Cryptophoto/CryptophotoUser/CryptoPhotoUtils.php")) {
      $this->path = APP_ROOT . "application/modules/mail/plugins/Cryptophoto/CryptophotoUser/CryptoPhotoUtils.php";
    }
    
    $this->log->debug("dirname: " . dirname(__FILE__) . "/CryptoPhotoUtils.php");
    $this->log->debug("root: " . APP_ROOT . "application/modules/mail/plugins/Cryptophoto/CryptophotoUser/CryptoPhotoUtils.php");
    $this->log->debug("path: " . $this->path);
    
    $this->_pluginDescription = "User Management component for Cryptophoto";
    
    $settings = NULL;
    
    if ($this->dbValid("#__cryptophotokeys")) {
      $db = Zend_Registry::get("dbAdapter");
      $db_values = $db->fetchAll("SELECT * FROM `#__cryptophotokeys`");
      $settings = $db_values[0];
    } 
    else {
      require_once ("library/jQuery/jQuery.php");
      jQuery::addMessage("Cryptophoto is not properly installed or configured");
      return;
    }
    
    // cryptophoto settings - keys and private salt
    $this->publickey  = $settings['publickey'] ? $settings['publickey'] : "";
    $this->privatekey = $settings['privatekey'] ? $settings['privatekey'] : "";
    $this->salt       = $settings['salt'] ? $settings['salt'] : "";
    $this->server     = $settings['server'] ? $settings['server'] : "https://cryptophoto.com";
    $this->log = Zend_Registry::get('log');
  }
  
  /*
   * menu item for the user's Cryptophoto Settings menu item
  */
  public function renderMailSettingsNav() {
    
    $params = array("title" => "My Cryptophoto", "text" => "Cryptophoto", "icon" => "images/cryptophoto.png");
    
    $class = get_class($this);
    $id = strtolower($class) . "_settings";
    
    $style = '';
    if (isset($params['icon'])) {
      $path = $this->_makeUrl($params['icon']);
      $style = "background: url($path) no-repeat 13px;display:block;height:21px;overflow:hidden;padding:11px 8px 6px 37px;white-space:nowrap;width:132px;color: #424242;font-size: 13px;padding: 13px 8px 8px 39px !important;text-shadow: 0 1px 0 #FFF !important;font-weight:bold";
    }
    
    $baseUrl = $this->_getModuleBaseUrl();
    
    echo "<li id=\"$id\"><a href=\"$baseUrl/plugininterface/index/$class/settings\" title=\"{$params['title']}\"><span style=\"$style\">{$params['text']}</span></a></li>";
  }
  
  /*
   * Render the user's CP settings
  */
  public function settings() {
    $this->_initView();
    echo $this->_gen_view_settings();
  }
  
  /*
   * Save a certain user's Cryptophoto settings, when changed
  */
  public function saveSettings($settings=array()) {
    $this->_initView();
    $userData = Zend_Auth::getInstance()->getStorage()->read();
    
    //get the user's id
    $this->userid = $userData['Account'];
    
    $request = $this->getRequest()->getParams();
    $this->log->debug("REQUEST=\n" . print_r($request, true));
    
    require_once ("library/jQuery/jQuery.php");
    
    // check for valid CP database
    if ($this->dbValid("#__cryptophoto")) {
      
      $db = Zend_Registry::get("dbAdapter");
      $db_values = $db->fetchAll("SELECT * FROM `#__cryptophoto` WHERE account = ? ", $this->userid);
      
      // the user exists in the CP database
      if (count($db_values)) {
        
        // valid requests for saving new settings
        if (!empty($request['save'])) {
          
          // enabled user's Cryptophoto
          if (!empty($request['enableCpCheckbox'])) {
            $db->query('UPDATE `#__cryptophoto` SET enabledCP = 1 WHERE account = ?', array($this->userid));
          }
          
          // disable user's Cryptophoto
          else {
            $db->query('UPDATE `#__cryptophoto` SET enabledCP = 0 WHERE account = ?', array($this->userid));
          }
          
          // success
          jQuery::addMessage("The settings have been updated");
        }
        
        // request was made with empty 'save' parameter
        else {
          $this->view->error = "Invalid request";
        }
      }
      
      // user not found in CP table though it should be - corrupted installation or files/db records were removed manually
      else {
        $this->view->error = "Something went wrong. This user does not exist in the Cryptophoto database";
      }
    }
    
    // cryptophoto table not found - admin plugin not installed or incmplete/corrupted installation
    else {
      $this->view->error = "The settings could not be updated. Cryptophoto is not properly installed or configured";
    }
    
    // render the CP widget
    echo $this->view->render("jsonresponse.phtml", null, true);
  }
  
  /*
   * Save a certain user's Cryptophoto settings, when changed
  */
  private function _gen_view_settings() {
    $userData = Zend_Auth::getInstance()->getStorage()->read();
    
    // get the user's id
    $this->userid = $userData['Account'];
    
    $request = $this->getRequest()->getParams();

    // properly installed Cryptophoto
    if ($this->dbValid("#__cryptophoto") && $this->privatekey && $this->publickey && $this->salt && $this->userid) {
      
      require_once ($this->path);
      
      // start a new Cryptophoto session
      $cp = new CryptoPhotoUtils($this->privatekey, $this->publickey, md5($this->salt . $this->userid), $this->server);
      
      $answer = $cp->start_session($_SERVER['REMOTE_ADDR'], array('interstitial_page_ops'=>1|2));
      
      if ($answer[0] == TRUE) {
        
        $this->view->gen_widget = $cp->get_gen_widget();
        $this->view->image_url = $this->_makeUrl("images/loading.gif");
        
        $db = Zend_Registry::get("dbAdapter");
        $db_values = $db->fetchAll("SELECT * FROM `#__cryptophoto` WHERE account = ?", array($this->userid));
        
        $this->view->enabledCP = 0;
        
        // user was found
        if (count($db_values)) {
          $enabledCP = $db_values[0]["enabledCP"];
          if ($enabledCP) {
            $this->view->enabledCP = 1;
          }
        } 
        else {
          
          // insert the user's details into the Cryptophoto table
          $db->query('INSERT INTO `#__cryptophoto` (`account`,`enabledCP`) VALUES(?,0)', array($this->userid));
        }
      } 
      else {
        $this->view->error = "Cryptophoto is not properly configured yet. Please contact your administrator.";
        $this->log->debug("Cryptophoto Server has responded with the following error message: " . $answer[1]);
      }
    }
    
    // cryptophoto table was not found - was deleted manually or installation didn't work well
    else {
      $this->view->error = "Cryptophoto is not properly configured yet. Please contact your administrator.";
      $error = 1;
    }
    
    if (isset($request['json'])) {
      header('Content-Type: application/x-javascript');
      $status = array('enabledCP' => $this->view->enabledCP == 1 ? true : false, 'cpscript' => $cp->get_gen_widget());
      $json = Zend_Json::encode($status);
      echo $json;
    } else {
      return $this->view->render("settings.phtml");
    }
  }
  
  /*
   * Method for capturing a successful login event
  */
  public function loginSuccess() {
    $request = $this->getRequest()->getParams();
    $this->log->debug(print_r($request, true));
    $this->userid = $request['email'];
    

    if ($request['expectJson'] == 1) {
//      return;
    }

    if (!$this->userid) {
      $this->userid = $request['emailName'] . '@' . $request['emailDomain'];
    }
    
    if ($this->dbValid("#__cryptophoto")) {
      $db = Zend_Registry::get("dbAdapter");
      $db_values = $db->fetchAll("SELECT * FROM `#__cryptophoto` WHERE account = ?", array($this->userid));
      
      // user exists in the Crpytophoto database
      if (count($db_values)) {
        
        // get the user's CP status
        
        if ($request['cpn'] && $request['cpt'] && $request['cpn'] == md5($this->userid . $request['password'] . $this->salt . $request['cpt'])) {
          return;
        }
        
        $enabledCP = $db_values[0]["enabledCP"];
        
        // Cryptophoto is enabled
        if ($enabledCP) {
          $auth = Zend_Auth::getInstance();
          $auth->clearIdentity();
          
          // save the user's password, to be able to differentiate between a guest or a logged-in user when displaying the CP authentication page
          $db->query("UPDATE `#__cryptophoto` SET password=?, mailserver=?, expire=? WHERE account = ?", array($request['password'], $request['requestedServer'], time() + 10 * 60, $this->userid));
          
          //redirect the user to the Cryptophoto authentication page
          $r = Zend_Controller_Action_HelperBroker::getStaticHelper('redirector');

          if ($request['expectJson'] == 1) {
            $r->gotoUrl('/mail/plugininterface/index/Cryptophoto_CryptophotoUser_Plugin/cpjson/id/' . $this->userid);
            return;
          }

          $r->gotoUrl('/mail/plugininterface/index/Cryptophoto_CryptophotoUser_Plugin/cpauth/id/' . $this->userid);
          return;
        }
      }
      
      // user not found in the Cryptophoto table
      else {
        $this->view->error = "Something went wrong with your Cryptophoto settings";
      }
    }
    
    // Cryptophoto table not found - invalid or corrupted installation
    else {
      $this->view->error = "Cryptophoto is not properly installed or configured";
    }
  }

  public function cpjson() {
$this->log->debug("*** IN CPJSON");
    $request = $this->getRequest()->getParams();

$this->log->debug(print_r($request, true));

    $id = $request['id'];

    $url = '/mail/plugininterface/index/Cryptophoto_CryptophotoUser_Plugin/cpauth/id/' . $id . "/json/1";
    $status = array('auth' => 'redirect', 'url' => $url);
    $json = Zend_Json::encode($status);

    header('Content-Type: application/x-javascript');
    echo $json;

    return;
  }
  
  /*
   * Cryptophoto authentication
  */
  public function cpauth() {
    $this->_initView();
    $request = $this->getRequest()->getParams();
    $this->log->debug("REQUEST=\n" . print_r($request, true));
    
    // get the current user's id
    $this->userid = $request['id'];
    $db = Zend_Registry::get("dbAdapter");
    $db_values = $db->fetchAll("SELECT * FROM `#__cryptophoto` WHERE account = ?", array($this->userid));
    
    if (isset($request['json']) && $request['json'] == 1) {
      $this->view->json = 1;
    } else {
      $this->view->json = 0;
    }


    // get the user's password - the password is saved if the user arrives to the CP auth page after a successful login
    if (count($db_values)) {
      
      // if empty - redirect to the original login page - it means the user reach the page through URL
      if (!$db_values[0]["password"]) {
        $r = Zend_Controller_Action_HelperBroker::getStaticHelper('redirector');
        $r->gotoUrl('/mail/');
        return;
      } 
      else {
        $expire = $db_values[0]["expire"];
        if (time() > $expire) {
          $db->query("UPDATE `#__cryptophoto` SET password='', mailserver='', expire='0' WHERE account = ?", array($this->userid));
          $r = Zend_Controller_Action_HelperBroker::getStaticHelper('redirector');
          $r->gotoUrl('/mail/');
          return;
        } 
        else {
          $db->query("UPDATE `#__cryptophoto` SET expire=? WHERE account = ?", array(time() + 10 * 60, $this->userid));
        }
      }
    }
    
    // user not in Cryptophoto table or missing Cryptophoto table - deleted or bad installation)  - normally, this should never be reached
    else {
      $r = Zend_Controller_Action_HelperBroker::getStaticHelper('redirector');
      $r->gotoUrl('/mail/');
      return;
    }
    
    // valid user
    if ($this->userid) {
      require_once ($this->path);
      $error = 0;
      
      // check for attempts to post empty values - don't even post to CP  if empty
      if (!empty($request['authenticate']) || !empty($request['authenticate'])) {
        
        // instantiate a new Cryptophoto object
        $authcp = new CryptoPhotoUtils($this->privatekey, $this->publickey, md5($this->salt . $this->userid), $this->server);
        
        // check for non-empty row and column codes
        if ((!empty($request["token_response_field_row"]) && !empty($request["token_response_field_col"])) || (!empty($request["cp_phc"]))) {
          
          $selector = $request["token_selector"];
          $row_code = $request["token_response_field_row"] ? $request["token_response_field_row"] : "";
          $col_code = $request["token_response_field_col"] ? $request["token_response_field_col"] : "";
          $cp_phc = $request["cp_phc"] ? $_POST["cp_phc"] : "";
          $ip = $_SERVER['REMOTE_ADDR'];
          
          // send codes to CP for authentication
          $authcp->verify_response($selector, $row_code, $col_code, $cp_phc, $ip);
          
          // codes ok. let's finally login this user
          if ($authcp->is_valid) {
            if ($this->dbValid("#__cryptophoto")) {
              $db = Zend_Registry::get("dbAdapter");
              $db_values = $db->fetchAll("SELECT * FROM `#__cryptophoto` WHERE account = ?", array($this->userid));
              
              // user found
              if (count($db_values)) {
                
                // save the user's password,
                $db->query("UPDATE `#__cryptophoto` SET password='', mailserver='' WHERE account = ?", array($this->userid));
                $account = $db_values[0]['account'];
                list($emailName, $emailDomain) = explode("@", $account);

                $this->view->emailName = $emailName;
                $this->view->emailDomain = $emailDomain;
                $this->view->mailserver = $db_values[0]['mailserver'];
                $this->view->password = $db_values[0]['password'];
                $this->view->time = time();
                $this->view->json = $request['json'] == 1 ? 1: 0;
                $this->view->nonce = md5($account . $db_values[0]['password'] . $this->salt . $this->view->time);
              }
              
              // user not found in the Cryptophoto table
              else {
                $this->view->error = "Something went wrong with your Cryptophoto settings";
                $error = 1;
              }
              
              return $this->view->render("autopost.phtml");
            }
          }
          
          // Cryptophoto authenticaion failed
          else {
            $this->view->error = "Invalid codes. Try again!";
            $error = 1;
          }
        }
        
        // provided codes are empty
        else {
          $this->view->error = "The codes may not be empty!";
        }
      }
      
      $db = Zend_Registry::get("dbAdapter");
      $db_values = $db->fetchAll("SELECT * FROM `#__cryptophoto` WHERE account = ?", array($this->userid));
      
      if (count($db_values)) {
        
        // new Cryptophoto object and sessoin
        $cp = new CryptoPhotoUtils($this->privatekey, $this->publickey, md5($this->salt . $this->userid), $this->server);
        $answer = $cp->start_session($_SERVER['REMOTE_ADDR'],array('authentication'=>'true'));
        
        if ($answer[0] == TRUE) {
          
          // get the user's CP widget
          $this->view->gen_auth = $cp->get_auth_widget();
          $this->view->image_url = $this->_makeUrl("images/loading.gif");
        } 
        else {
          $this->view->error = "Something went terribly wrong with your Cryptophoto settings";
          $this->log->debug("Cryptophoto Server has responded with the following error message: " . $answer[1]);
        }
      }
      
      // user not found in CP table or the table itself is missing
      else {
        $this->view->error = "Something went terribly wrong with your Cryptophoto settings";
        $error = 1;
      }
      
      return $this->view->render("cpauth.phtml");
    }
    
    // user not found or CP table missing
    else {
      $r = Zend_Controller_Action_HelperBroker::getStaticHelper('redirector');
      $r->gotoUrl('/mail/');
      return;
    }
  }
  
  /*
   * authRequired
  */
  public function authRequired($method) {
    if ($method == "cpauth" || $method == "cpjson") {
      return false;
    }
    return true;
  }
  
  /*
   * checks that the Cryptophoto table exists
  */
  public function dbValid($tableName) {
    
    $db = Zend_Registry::get("dbAdapter");
    $db_values = $db->fetchAll("SHOW TABLES LIKE '" . $tableName . "'");
    if (count($db_values)) {
      return true;
    } 
    else {
      return false;
    }
  }
}
